# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-22 08:08
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cc', '0008_withdrawtransaction_state'),
    ]

    operations = [
        migrations.AlterField(
            model_name='withdrawtransaction',
            name='txid',
            field=models.CharField(blank=True, db_index=True, max_length=100, null=True, verbose_name='Txid'),
        ),
    ]
