# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-22 08:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cc', '0007_withdrawtransaction_state'),
    ]

    operations = [
        migrations.AlterField(
            model_name='withdrawtransaction',
            name='state',
            field=models.CharField(choices=[('NEW', 'New'), ('ERROR', 'Error'), ('DONE', 'Done')], default='NEW', max_length=10, verbose_name='State'),
        ),
    ]
